﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Trifork.SmartFraming.HL7.Api;

namespace Trifork.SmartFraming.HL7.TestClient
{
    public class AuthenticateUserByIdTokenContextAgent: IContextAgent
    {
        public const string SUBJECT = "AuthenticateUser";
	    const string AUTHENTICATEUSER_IN_EXPECTEDUSERLOGON_OIOIDWSH = "AuthenticateUser.in.ExpectedUserLogon.Oioidws-h";
	    const string AUTHENTICATEUSER_OU_STATUS = "AuthenticateUser.Ou.Status";
	    const string AUTHENTICATEUSER_OU_LOGON_OIOIDWS = "AuthenticateUser.Ou.Logon.Oioidws-h";
	    const string AUTHENTICATEUSER_OU_NAME = "AuthenticateUser.Ou.Name";
	    const string AUTHENTICATEUSER_OU_SOSIID = "AuthenticateUser.Ou.Sosiid";

        private readonly LoginUser user;
        private readonly TextBox _textboxForOutput;

        private static Dictionary<String, String> CreateActionResultMap(String[] itemNames, String[] itemValues)
        {
            var map = new Dictionary<String, String>();

            for (var i = 0; i < itemNames.Length; i++)
            {
                map.Add(itemNames[i], itemValues[i]);
            }

            return map;
        }

        public AuthenticateUserByIdTokenContextAgent(LoginUser user, TextBox textboxForOutput)
        {
            this.user = user;
            _textboxForOutput = textboxForOutput;    
        }

        #region IContextAgent Members

        public ActionResult contextChangesPending(int coupon, string[] itemNames, string[] itemValues)
        {
            var actionResult = new ActionResult();

            try
            {
                var map = CreateActionResultMap(itemNames, itemValues);
                var currentUserCpr = map[AUTHENTICATEUSER_IN_EXPECTEDUSERLOGON_OIOIDWSH];

                var idCard = SOSIUtil.GetIdCard(user);
                var idToken = SOSIUtil.GetIdentityTokenFromIDCard(idCard, user.UserCertificateSerialNo);

                actionResult.outputNames = new[]
                                               {
                                                   AUTHENTICATEUSER_OU_STATUS,
                                                   AUTHENTICATEUSER_OU_LOGON_OIOIDWS,
                                                   AUTHENTICATEUSER_OU_NAME,
                                                   AUTHENTICATEUSER_OU_SOSIID
                                               };
                
                actionResult.outputValues = new[] { 
						"Pass",
						"",
						"",
						idToken
				};
            } 
            catch (Exception e)
            {
                _textboxForOutput.AppendText(
                    "Exception in AuthenticateUserByIdTokenContextAgent:contextChangesPending: " + e);
            }

            return actionResult;
        }

        #endregion
    }
}
