﻿using System;
using System.IO;
using System.Security.Cryptography.X509Certificates;
using System.Xml.Linq;
using Nsi.Security;
using SDSD.SealApi;

namespace Trifork.SmartFraming.HL7.TestClient
{
    public class SOSIUtil
    {
        public static XElement GetIdCard(LoginUser user)
        {
            var card101 = new DGWSCard101Type
            {
                CardLifeTime = CardLifeTimeType.Hours24,
                IDCardType = CardType.user,
                AuthenticationLevel = 4,
                Issuer = "WinPLC",

                ITSystemName = "SOSITEST",
                OrganisationIDFormat = "medcom:cvrnumber",
                OrganisationID = user.CvrNr,
                OrganisationName = "A-Data ApS.",

                NameID = user.CprNr,
                CivilRegistrationNumber = user.CprNr,
                EmailAddress = "min.email@adatatest.com",
                GivenName = "Opdaterende",
                SurName = "Person",
                Occupation = "Læge",
                Role = "Læge",
                AuthorizationCode = "19901",
            };

            card101.Sign(GetCertificate(user.UserCertificateSerialNo));
            var signedCard = IDP.CallIdp(card101, "http://fmk-online.dk",
                                     "http://pan.certifikat.dk/sts/services/SecurityTokenService");
            return signedCard;
        }

        private static X509Certificate2 GetCertificate(string certificateSerialNo)
        {
            var certificateStore = new X509Store("My", StoreLocation.CurrentUser);
            certificateStore.Open(OpenFlags.ReadOnly);

            var certs = certificateStore.Certificates.Find(X509FindType.FindBySerialNumber, certificateSerialNo, true);
            certificateStore.Close();

            if (certs.Count == 0)
            {
                throw new ArgumentException("Ugyldigt CertificateSerialNo");
            }
            return certs[0];
        }

        internal static string GetIdentityTokenFromIDCard(XElement idCard, string certificateSerialNo)
        {
            using (MemoryStream ms = HttpPostClient.CallSTS(idCard,
                "http://niab01.nsp-test.netic.dk:8080/sts/services/IdentityTokenService",
                "http://fmk-online.dk"))
            {
                return Converter.StreamAssertionCBase64(ms);
            }
        }
    }
}
