﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Configuration;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Security.Permissions;
using System.Windows.Forms;
using System.Xml.Linq;
using SDSD.SealApi;
using Trifork.SmartFraming.HL7.Api;

namespace Trifork.SmartFraming.HL7.TestClient
{
    [PermissionSet(SecurityAction.Demand, Name = "FullTrust")]
    public partial class SmartframeForm : Form
    {
        private int coupon;
        private TestClientConfig clientConfig;
        private bool eventHandlersAreSet = false;
        private SmartFrame smartFrame;
        XElement signedCard = null;

        public SmartframeForm()
        {
            InitializeComponent();

            clientConfig = ConfigurationManager.GetSection("testConfig") as TestClientConfig;
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            comboBoxUrl.DisplayMember = "Url";
            comboBoxUrl.DataSource = new List<UrlConfig>(clientConfig.Urls.OfType<UrlConfig>()); 

            comboBoxCert.DisplayMember = "Username";
            comboBoxCert.DataSource = new List<UserConfig>(clientConfig.Users.OfType<UserConfig>());

            comboBoxOrgType.DataSource = new List<OrganizationConfig>(clientConfig.Organizations.OfType<OrganizationConfig>());
            comboBoxOrgType.DisplayMember = "Name";
            comboBoxOrgType.SelectedIndex = 0;
            comboBoxCPR.SelectedIndex = 0;
        }

        XElement GetIdCard(LoginUser user)
        {
            if (signedCard != null)
            {
                return signedCard;
            }

            try
            {
                signedCard = SOSIUtil.GetIdCard(user);
            }
            catch (Exception e)
            {
                MessageBox.Show("Fejl ved login med SOSI ID-kort: " + e.Message, "SOSI ID Kort Login");
            }

            return signedCard;
        }

        private void buttonShow_Click(object sender, EventArgs e)
        {
            if (smartFrame != null)
            {
                panelFrame.Controls.Remove(smartFrame);
            }

            var selectedUrl = (UrlConfig)comboBoxUrl.SelectedItem;

            smartFrame = new SmartFrame(string.IsNullOrEmpty(selectedUrl.SosiLoginUrl) ? SmartFrame.LoginMethod.SmartframeLogin_None :  SmartFrame.LoginMethod.SmartframeLogin_SOSIIdKort);
            panelFrame.Controls.Add(smartFrame);
            smartFrame.Dock = DockStyle.Fill;

            coupon = smartFrame.JoinCommonContext(new MyContextParticipant(smartFrame.ContextData, textBoxChanges),
                                                  "Smartframe Testclient");
            smartFrame.JoinCommonContext(new MyContextAgent(textBoxChanges), "JournalNotat");
            smartFrame.JoinCommonContext(new PDFContextAgent(), "FMK_PDF_Response");

            if (!eventHandlersAreSet)
            {
                comboBoxCPR.SelectedIndexChanged += new EventHandler(comboBoxCPR_SelectedIndexChanged);
                comboBoxOrgType.SelectedIndexChanged += new EventHandler(comboBoxOrgType_SelectedIndexChanged);
                comboBoxOnBehalfOf.SelectedIndexChanged += comboBoxOnBehalfOf_SelectedIndexChanged;
                eventHandlersAreSet = true;
            }

            var user = comboBoxCert.SelectedItem as UserConfig;

            if (user != null)
            {
                var loginuser = new LoginUser()
                                            {
                                                CprNr = user.CprNr,
                                                CvrNr = user.CvrNr,
                                                UserCertificateSerialNo = user.CertificateSerialNo,
                                                Username = user.Username
                                            };

                loginuser.SignedIDCard = GetIdCard(loginuser).ToString();

                smartFrame.JoinCommonContext(new AuthenticateUserByIdTokenContextAgent(loginuser, textBoxChanges), AuthenticateUserByIdTokenContextAgent.SUBJECT);


                var org = comboBoxOrgType.SelectedItem as OrganizationConfig;
                smartFrame.ContextData.SetItemValues(
                    new[]
                        {
                            TestHL7Helper.Patient_Id_IdListName,
                            TestHL7Helper.UserOrganization_Id_Type, TestHL7Helper.UserOrganization_Id_Number,
                            TestHL7Helper.UserOrganization_Id_SorId,
                            TestHL7Helper.UserOrganization_Id_OnBehalfOfPersonId,
                        },
                    new[]
                        {
                            TestHL7Helper.GetPatient_Id_IdListValue(SelectedCpr()), org.Type, org.Id, org.SorId, TestHL7Helper.GetPatient_Id_IdListValue(SelectedOnBehalfOfCpr())
                        }, coupon);
                smartFrame.EndContextChanges(coupon);
                smartFrame.PublishChangesDecision(coupon, Decision.Accept);

                if (!string.IsNullOrEmpty(selectedUrl.SosiLoginUrl))
                    smartFrame.Navigate(selectedUrl.Url, selectedUrl.SosiLoginUrl, coupon, loginuser);
                else
                {
                    smartFrame.Navigate(selectedUrl.Url, coupon);
                }
            }
        }

        void comboBoxCPR_SelectedIndexChanged(object sender, EventArgs e)
        {
          smartFrame.ContextData.SetItemValues(new[] { TestHL7Helper.Patient_Id_IdListName }, new[] { TestHL7Helper.GetPatient_Id_IdListValue(SelectedCpr()) }, coupon);
          smartFrame.EndContextChanges(coupon);
          smartFrame.PublishChangesDecision(coupon, Decision.Accept);
        }

       

        string SelectedCpr()
        {
            if (comboBoxCPR.SelectedIndex == -1)
            {
                comboBoxCPR.SelectedIndex = 0;
            }

            return ((string)comboBoxCPR.SelectedItem).Replace("-", "");
        }

        string SelectedOnBehalfOfCpr()
        {
          if (comboBoxOnBehalfOf.SelectedIndex == -1)
          {
            return "";
          }

          return ((string)comboBoxOnBehalfOf.SelectedItem).Replace("-", "");
        }

        void comboBoxOrgType_SelectedIndexChanged(object sender, EventArgs e)
        {
          if (smartFrame != null)
          {
            var org = comboBoxOrgType.SelectedItem as OrganizationConfig;
            smartFrame.ContextData.SetItemValues(
              new[]
                {
                  TestHL7Helper.UserOrganization_Id_Type, TestHL7Helper.UserOrganization_Id_Number,
                  TestHL7Helper.UserOrganization_Id_SorId
                }, new[] {org.Type, org.Id, org.SorId}, coupon);
            smartFrame.EndContextChanges(coupon);
            smartFrame.PublishChangesDecision(coupon, Decision.Accept);
          }
        }

        private void comboBoxOnBehalfOf_SelectedIndexChanged(object sender, EventArgs e)
        {
            smartFrame.ContextData.SetItemValues(new[] { TestHL7Helper.UserOrganization_Id_OnBehalfOfPersonId },
                                                 new[] { TestHL7Helper.GetPatient_Id_IdListValue(SelectedOnBehalfOfCpr()) }, coupon);
            smartFrame.EndContextChanges(coupon);
            smartFrame.PublishChangesDecision(coupon, Decision.Accept);
        }

        private void buttonContextAction_Click(object sender, EventArgs e)
        {
            var result = smartFrame.Perform(coupon, new[] { "TestAction.In.Message" }, new[] { "Test af contextAction OK" });
            if (result != null && result.outputValues != null && result.outputValues.Length > 0 && result.outputNames != null && result.outputNames.Length > 0)
            {
                MessageBox.Show(string.Format("outputNames[0]: {0}\noutputValues[0] {1}", result.outputNames[0],
                                              result.outputValues[0]), "Besked fra værts-applikation");
            }
            else
            {
                MessageBox.Show("Intet resultat returneret", "Besked fra værtsapplikation");
            }
        }

    
        private void comboBoxCert_SelectedIndexChanged(object sender, EventArgs e)
        {
          if(comboBoxCert.SelectedIndex >= 0)
          {
            var cert = (UserConfig) comboBoxCert.SelectedItem;
            var onBehalfList = new List<string>() {""};
            if(!string.IsNullOrEmpty(cert.OnBehalfOf1))
            {
              onBehalfList.Add(cert.OnBehalfOf1.Substring(0, 6) + "-" + cert.OnBehalfOf1.Substring(6,4));
            }
            if(!string.IsNullOrEmpty(cert.OnBehalfOf2))
            {
              onBehalfList.Add(cert.OnBehalfOf2.Substring(0, 6) + "-" + cert.OnBehalfOf2.Substring(6, 4));
            }
            if(!string.IsNullOrEmpty(cert.OnBehalfOf3))
            {
              onBehalfList.Add(cert.OnBehalfOf3.Substring(0, 6) + "-" + cert.OnBehalfOf3.Substring(6, 4));
            }
            if(!string.IsNullOrEmpty(cert.OnBehalfOf4))
            {
              onBehalfList.Add(cert.OnBehalfOf4.Substring(0, 6) + "-" + cert.OnBehalfOf4.Substring(6, 4));
            }
            comboBoxOnBehalfOf.DataSource = onBehalfList;
          }
          else
          {
            comboBoxOnBehalfOf.DataSource = null;
            comboBoxOnBehalfOf.SelectedItem = -1;
          }
        }

        private void button1_MouseClick(object sender, MouseEventArgs e)
        {
            smartFrame._browser.CallScript(

                "var inputs = document.getElementsByTagName('input'); " +
                "for(var i = 0; i <inputs.length; i++) { " +
                "if(inputs[i].parentNode.className == 'gwt-DropDownListBox drugComboBox drug-combobox') { var node = inputs[i]; alert(node.parentNode.innerHTML); node.onkeydown= function() { alert('keydown'); }; } };");
        }

        private void buttonPDF_Click(object sender, EventArgs e)
        {
            var result = smartFrame.Perform(coupon, new[] { "FMK_PDF_Request.in.PatientPersonIdentifier" }, new[] { SelectedCpr() });
            
        }
    }
}
